/* Imposta il datestyle secondo il formato europeo gg/mm/aaaa */
set DateStyle='SQL, DMY';

/* Creazione tabelle */

create table CollaboratoreFisso
(
    CF           char(16) primary key,
    Nome         varchar(30) not null,
    Indirizzo    varchar(30),
    Telefono     numeric(10)
);

create table CollaboratoreOccasionale
(
    CF           char(16) primary key,
    Nome         varchar(30) not null,
    Indirizzo    varchar(30),
    Telefono     numeric(10)
);

create table MembroDelComitatoDiRedazione
(
    CF                  char(16) primary key
                            references CollaboratoreFisso(CF),
    Ruolo               varchar(12) not null,
    AreaDiCompetenza    varchar(20),
    check ((Ruolo = 'Supervisore' and AreaDiCompetenza is not null) or
          (Ruolo = 'Direttore' and AreaDiCompetenza is null))
);

create table PercepisceStipendio
(
    CF            char(16)
                      references CollaboratoreFisso(CF),
    Importo       numeric(6, 2),
    DataInizio    date,
    DataFine      date,
    primary key (CF, Importo, DataInizio, DataFine)
);

create table PercepisceOnorario
(
    CF            char(16)
                      references CollaboratoreOccasionale(CF),
    Importo       numeric(6, 2),
    Data          date,
    primary key (CF, Importo, Data),
    unique (Importo, Data)
);

create table Periodico
(
    Titolo            varchar(30) primary key,
    Tipo              varchar(12) not null, /* TODO: creare un dominio apposito? */
    NumeroArticoli    numeric(10) default 0
);

create table Lavora
(
    CF                 char(16)
                           references CollaboratoreFisso(CF),
    TitoloPeriodico    varchar(30)
                           references Periodico(Titolo),
    primary key (CF, TitoloPeriodico)
);

create table Redazione
(
    TitoloPeriodico    varchar(30) primary key
                           references Periodico(Titolo),
    Indirizzo          varchar(30),
    Telefono           numeric(10)
);

create table Numero
(
    DataDiUscita       date,
    TitoloPeriodico    varchar(30)
                           references Periodico(Titolo),
    primary key (DataDiUscita, TitoloPeriodico)
/* TODO: check un mensile non pu uscire prima di un mese del precedente, etc. */
);

create table Articolo1
(
    Codice    numeric(6)  primary key,
    Titolo    varchar(50) not null
);

create table Articolo2
(
    Codice    numeric(6)  primary key
                      references Articolo1(Codice),

    Pagina        numeric(3),
    PosizioneX    numeric(3),
    PosizioneY    numeric(3),
    Area          varchar(20) not null,

    DataDiUscita       date,
    TitoloPeriodico    varchar(30),
    foreign key (DataDiUscita, TitoloPeriodico)
                references Numero(DataDiUscita, TitoloPeriodico),

    CFSupervisore     char(16)
                          references MembroDelComitatoDiRedazione(CF)

    /* vincolo V1 - req SQL-99
    check ( Area = ( select AreaDiCompetenza
                     from MembroDelComitatoDiRedazione
                     where CF = CFSupervisore )
          )*/
);

create table Scrive1
(
    CodiceArticolo    numeric(6)
                          references Articolo1(Codice),
    CF                char(16)
                          references CollaboratoreFisso(CF),
    primary key (CodiceArticolo, CF)
);

create table Scrive2
(
    CodiceArticolo    numeric(6)
                          references Articolo1(Codice),
    CF                char(16)
                          references CollaboratoreOccasionale(CF),
    primary key (CodiceArticolo, CF)
);

create table Per
(
    CodiceArticolo    numeric(6) primary key
                          references Articolo1(Codice),
    Importo           numeric(6, 2),
    Data              date,
    foreign key (Importo, Data)
                references PercepisceOnorario(Importo, Data)
);

create table Editoriale
(
    Codice        numeric(6) primary key,
    Titolo        varchar(50) not null,

    Pagina        numeric(3),
    PosizioneX    numeric(3),
    PosizioneY    numeric(3),

    DataDiUscita       date,
    TitoloPeriodico    varchar(30),
    foreign key (DataDiUscita, TitoloPeriodico)
                references Numero(DataDiUscita, TitoloPeriodico),

    CFAutore     char(16) not null
                     references MembroDelComitatoDiRedazione(CF)
);

create table SpazioPubblicitario
(
    Codice        numeric(6) primary key,
    Acquirente    varchar(30) not null,
    Prezzo        numeric(8, 2) not null,

    Pagina        numeric(3),
    PosizioneX    numeric(3),
    PosizioneY    numeric(3),

    DataDiUscita       date,
    TitoloPeriodico    varchar(30),
    foreign key (DataDiUscita, TitoloPeriodico)
                references Numero(DataDiUscita, TitoloPeriodico)
);


/* Inserimenti per popolare la base di dati */

insert into periodico values ('Papersera', 'quotidiano');
insert into periodico values ('L''eco di Paperopoli', 'quotidiano');
insert into periodico values ('Topolinia News', 'settimanale');
insert into periodico values ('Ducklair Reports', 'quindicinale');
insert into periodico values ('Papera moderna', 'mensile');

insert into numero values ('01/07/2006', 'Papersera');
insert into numero values ('02/07/2006', 'Papersera');
insert into numero values ('03/07/2006', 'Papersera');
insert into numero values ('04/07/2006', 'Papersera');
insert into numero values ('05/07/2006', 'Papersera');
insert into numero values ('06/07/2006', 'Papersera');
insert into numero values ('07/07/2006', 'Papersera');
insert into numero values ('01/07/2006', 'L''eco di Paperopoli');
insert into numero values ('02/07/2006', 'L''eco di Paperopoli');
insert into numero values ('03/07/2006', 'L''eco di Paperopoli');
insert into numero values ('04/07/2006', 'L''eco di Paperopoli');
insert into numero values ('05/07/2006', 'L''eco di Paperopoli');
insert into numero values ('06/07/2006', 'L''eco di Paperopoli');
insert into numero values ('07/07/2006', 'L''eco di Paperopoli');
insert into numero values ('01/07/2006', 'Topolinia News');
insert into numero values ('02/07/2006', 'Ducklair Reports');
insert into numero values ('02/07/2006', 'Papera moderna');

insert into CollaboratoreFisso values ('DPPPRN67M08H501K', 'Paperon de Paperoni', 'via dei Castagni, 10', 177007422);
insert into CollaboratoreFisso values ('PPRPLN34R09H501O', 'Paolino Paperino', 'via dei Ciliegi, 20', 277780715);
insert into CollaboratoreFisso values ('PPRPRG64M02H501N', 'Paperoga', 'via dei Peri, 30', 3456188321);
insert into CollaboratoreFisso values ('PPRGTN48C06H501M', 'Gastone Paperone', 'via degli Aranci, 40', 3407264261);
insert into CollaboratoreFisso values ('DPPPCI61P24H501Z', 'Pico de Paperis', 'via delle Mele, 50', 3121105140);
insert into CollaboratoreFisso values ('PTGRHM52E16H501N', 'Archimede Pitagorico', 'via dei Meloni, 60', 3121105142);

insert into CollaboratoreFisso values ('RCKJND61B12H501I', 'John Rockerduck', 'via dei Peschi, 70', 105875226);
insert into CollaboratoreFisso values ('MCBBGT60P42H501Y', 'Brigitta McBridge', 'via degli Olivi, 80', 184700712);
insert into CollaboratoreFisso values ('PPRMLY61R68H501F', 'Emily Paperett', 'via degli Olmi, 90', 3246865312);

insert into MembroDelComitatoDiRedazione values ('DPPPRN67M08H501K', 'Direttore');
insert into MembroDelComitatoDiRedazione values ('DPPPCI61P24H501Z', 'Supervisore', 'Cronaca');
insert into MembroDelComitatoDiRedazione values ('PTGRHM52E16H501N', 'Supervisore', 'Cronaca');

insert into MembroDelComitatoDiRedazione values ('RCKJND61B12H501I', 'Direttore');
insert into MembroDelComitatoDiRedazione values ('PPRMLY61R68H501F', 'Supervisore', 'Cronaca');

insert into CollaboratoreOccasionale values ('MTRNLT52H20H501G', 'Anacleto Mitraglia', 'via dei Ciliegi, 50', 269330710);
insert into CollaboratoreOccasionale values ('SGNFLI70B11H501P', 'Filo Sganga', 'via dei Noci, 60', 242747311);
insert into CollaboratoreOccasionale values ('PPRSRZ64R25H501J', 'Sgrizzo Papero', 'via dei Salici, 70', 3163654360);

insert into PercepisceStipendio values ('DPPPRN67M08H501K', 0, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('PPRPLN34R09H501O', 10.50, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('PPRPRG64M02H501N', 10.50, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('PPRGTN48C06H501M', 2000.00, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('DPPPCI61P24H501Z', 800.00, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('PTGRHM52E16H501N', 800.00, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('RCKJND61B12H501I', 3100.40, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('MCBBGT60P42H501Y', 1400.00, '01/01/2006', '31/12/2008');
insert into PercepisceStipendio values ('PPRMLY61R68H501F', 1400.00, '01/01/2006', '31/12/2008');

/* insert into redazione [...]; */

insert into lavora values ('DPPPRN67M08H501K', 'Papersera');
insert into lavora values ('PPRPLN34R09H501O', 'Papersera');
insert into lavora values ('PPRPRG64M02H501N', 'Papersera');
insert into lavora values ('PPRGTN48C06H501M', 'Papersera');
insert into lavora values ('DPPPCI61P24H501Z', 'Papersera');
insert into lavora values ('PTGRHM52E16H501N', 'Papersera');

insert into lavora values ('RCKJND61B12H501I', 'L''eco di Paperopoli');
insert into lavora values ('MCBBGT60P42H501Y', 'L''eco di Paperopoli');
insert into lavora values ('PPRMLY61R68H501F', 'L''eco di Paperopoli');


/* Operazione 1
(esempio di articolo scritto da un collaboratore fisso) */

insert into Articolo1
values ('164823', 'La Banda Bassotti colpisce ancora');

insert into Articolo2
values ('164823', 6, 30, 40, 'Cronaca', '01/07/2006', 'Papersera', 'DPPPCI61P24H501Z');

insert into Scrive1 
values ('164823', 'PPRPLN34R09H501O');

update Periodico
set NumeroArticoli = ( select NumeroArticoli+1
                       from Periodico
                       where Titolo = 'Papersera' )
where Titolo = 'Papersera';

/* Operazione 1
(esempio di articolo scritto da un collaboratore occasionale) */

insert into Articolo1
values (264210, 'La strega Amelia ruba la Numero Uno');

insert into Articolo2
values (264210, 17, 50, 10, 'Cronaca', '01/07/2006', 'Papersera', 'PTGRHM52E16H501N');

insert into Scrive2
values (264210, 'MTRNLT52H20H501G');

update Periodico
set NumeroArticoli = ( select NumeroArticoli+1 
                       from Periodico
                       where Titolo = 'Papersera' )
where Titolo = 'Papersera';

insert into PercepisceOnorario
values ('MTRNLT52H20H501G', 100.50, '01/07/2006');

insert into Per
values (264210, 100.50, '01/07/2006');


/* Operazione2 -
Se si vuole solo registrare un nuovo numero l'operazione  banale: */

insert into numero values ('01/06/2006', 'Papersera');

/* Se in seguito si vogliono anche registrare gli elementi della composizione
si pu riccorrere all'operazione 1 e alle seguenti: */

insert into Editoriale 
values (241736, 'In questo numero del Papersera', 2, 1, 1, '01/07/2006', 'Papersera', 'DPPPRN67M08H501K');

insert into SpazioPubblicitario
values (930318, 'Gelati PdP', 10090, 4, 30, 15, '01/07/2006', 'Papersera');


/* Operazione 3 */

select A2.TitoloPeriodico, A2.DataDiUscita, A1.Titolo, A2.Pagina
from Articolo1 A1, Articolo2 A2
where A1.Codice = A2.Codice and
      (select extract(month from A2.DataDiUscita)) =
      (select extract(month from CURRENT_DATE))
order by TitoloPeriodico, DataDiUscita, Pagina;


/* Operazione 4 -
(la parola cercata  "Bassotti") */

select Nome
from CollaboratoreFisso
where CF in ( select distinct CF
              from Scrive1 S1, Articolo1 A1, Articolo2 A2
              where  A1.Codice = A2.Codice and
                     S1.CodiceArticolo = A2.Codice and
                     A1.Titolo like '%Bassotti%')
    union
select Nome
from CollaboratoreOccasionale
where CF in ( select distinct CF
              from Scrive2 S2, Articolo1 A1, Articolo2 A2
              where  A1.Codice = A2.Codice and
                     S2.CodiceArticolo = A2.Codice and
                     A1.Titolo like '%Bassotti%');


/* Operazione 5*/

select A.TitoloPeriodico, count (A.Codice)
from Articolo2 A, Scrive2 S
where (select extract(year from A.DataDiUscita)) =
      (select extract(year from CURRENT_DATE))
      and A.Codice = S.CodiceArticolo
group by A.TitoloPeriodico;


/* Operazione 6 */

select sum(Importo)
from PercepisceStipendio
where DataFine >= CURRENT_DATE;


/* Operazione 7 */

select sum(Importo)
from PercepisceOnorario
where (select extract(month from Data)) =
      (select extract(month from CURRENT_DATE))
  and (select extract(year from Data)) =
      (select extract(year from CURRENT_DATE));


/* Operazione 8 */

select sum(Prezzo)
from SpazioPubblicitario
where (select extract(month from DataDiUscita)) =
      (select extract(month from CURRENT_DATE))
  and (select extract(year from DataDiUscita)) =
      (select extract(year from CURRENT_DATE));


/* Operazione 9 */

select Nome
from CollaboratoreFisso
where CF in (select distinct CF
             from Scrive1 S, Articolo2 A
             where A.Codice = S.CodiceArticolo and 
                   (select extract(week from A.DataDiUscita)) =
                   (select extract(week from CURRENT_DATE)) and
                   (select extract(year from A.DataDiUscita)) =
                   (select extract(year from CURRENT_DATE)));


/* Operazione 10 -
La media si ottiene dividendo la somma degli articoli scritti per ciascun tipo di periodico con i numeri usciti
(conteggiati sempre in base alla tipologia del periodico) */

select Tipo, sum(NumeroArticoli) / ( select count(*)
                                     from Numero
                                     where TitoloPeriodico in ( select Titolo
                                                                from Periodico P2
                                                                where P2.Tipo = P.Tipo )
                                    ) as MediaArticoli 
from Periodico P
group by Tipo;
